<?php

$now = shell_exec("date '+%Y-%m-%d-%H-%M-%S-'");
$now = trim($now);

// Global test parameters.  This is the IP, APID, name and password to use.
$ip = "127.0.0.1";
$user = "linminapi";
$password = "baremetalapi";

// Test user_supplied_id to be used for all provisioning API tests
$user_supplied_id = "TestServer";

// Test values for adding and reading, all under the key $user_supplied_id
$node_name = "TestHost";
$node_domain = "TestDomain";
$mac_address = "00%3A0c%3A29%3A4f%3Ac7%3A23";
$node_ip_address = "192.168.0.200";
$node_subnet_mask = "255.255.255.0";
$node_nameserver = "68.94.156.1,68.94.157.1";
$node_default_gateway = "192.168.0.1";
$enable_provisioning_flag = "nextbootonly";
$force_pxe_ip = "on";

// Test values for updating, all under the key $user_supplied_id
$node_name_up = "TestHostUpdate";
$node_domain_up = "TestDomainUpdate";
$mac_address_up = "00%3A0c%3A29%3A4f%3Ac7%3A23";
$node_ip_address_up = "10.0.0.200";
$node_subnet_mask_up = "255.0.0.0";
$node_nameserver_up = "10.168.1.254";
$node_default_gateway_up = "10.0.0.1";
$enable_provisioning_flag_up = "nextbootonly";
$force_pxe_ip_up = "on";

// Return codes
$auth_failed = -1;
$templates_failed = -2;
$roles_failed = -3;
$add_failed = -4;
$read_failed = -5;
$modify_failed = -6;
$delete_failed = -7;


///////////////////////////////////////
// Input Data
///////////////////////////////////////

echo ("Enter a unique name for the machine being provisioned: ");
$node_name = fgets(STDIN);
echo ("Enter a MAC address using this format %3A4f%3Ac7%3A23: ");
$mac_address = "00%3A0c%3A29".fgets(STDIN);
echo ("Enter the last number of an IP address (192.168.0.): ");
$node_ip_address = "192.168.0.".fgets(STDIN);




///////////////////////////////////////
// Run Tests 
///////////////////////////////////////


// Get the globalAPID so tests can be run.
$globalApid = getGlobalAPID();

$apid = testAuth($ip, $globalApid, $user, $password);
if ( $apid == -1 ) {
	echo ("Could not get temporary APID for ip:".$ip." user:".$user." password:".$password."\n");
	echo ("Aborting tests\n");
	exit ($auth_failed);
}

$template = testTemplates($ip, $apid);
if ( $template == -1 ) {
	echo ("Could not get template #1.  You must create at least one template.\n");
	echo ("Aborting tests\n");
	exit ($templates_failed);
}

echo ("Adding new role Template is ".$template."\n");
if ( testAdd ($ip, $apid, $template) == -1 ) {
	echo ("Test add failed.  Aborting tests. \n");
	exit ($add_failed);
} else {
	echo ("Test add succeeded. \n");
}


exit (0);


///////////////////////////////////////
// Tests 
///////////////////////////////////////

/**
 * Given the global APID, user and password, request a 5 minute APID to be used for the unit
 * tests.  This function is a unit test as well.  If it fails, all other tests will be aborted.
 * @param The global APID
 * @param USer name
 * @param Passwrod
 * @result A valid APID or -1 if it failed.  Results are logged.
*/
function testAuth($ip, $globalApid, $user, $password) {

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&apid=$globalApid&auth_user=$user&auth_user_pw=$password&node_time_zone=Etc%2FGMT&enable_provisioning_flag=nextbootonly&rtn_format=NQV&action=authorize' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	logResult ($output);

	if (  successful($output) == 0) {
		$apid = getResultValue ($output, "apid" );
		echo ("APID is: ".$apid."\n");
		return $apid;
	}
	return "-1";
}


/**
 * Get one of the available templates to set this new role to.
*/
function testTemplates($ip, $apid) {

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&apid=$apid&rtn_format=NQV&action=templates' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	logResult ($output);
	
	echo ("Choose one of the following templates \n");
	print_r($output);
	echo ("Choose a template number: ");
	$number = fgets(STDIN);
	$number = rtrim($number);

	if (  successful($output) == 0) {
		$template = getResultValue ($output, "MAC_SPECIFIC_TEMPLATE_".$number."_title" );
		echo ("Template 1 is: ".$template."\n");
		return $template;
	}
	return "-1";
}

/**
 * Given the global APID, user and password, request a 5 minute APID to be used for the unit
 * tests.  This function is a unit test as well.  If it fails, all other tests will be aborted.
 * @param The global APID
 * @param USer name
 * @param Passwrod
 * @result A valid APID or -1 if it failed.  Results are logged.
*/
function testRoles($ip, $apid) {

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&apid=$apid&rtn_format=NQV&action=roles' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	logResult ($output);

	if (successful($output) == 0) {
		// We can't assume our test set is the first role in the list.
		// How can we find ourselves?
		// return testDataset( "Roles", $output, "MAC_SPECIFIC_ROLE_1_" );
		return 0;
	}
	return -1;
}


/**
 * Given the 5 minute APID and IP address, test a read of data for nickname "TestServer".
 * Results are logged.  Function returns -1 if failed, or 0 if success.
 * @param IP address
 * @param Five munite APID given for IP address.
 * @param noLogOnFail There is one case where read may fail due to the test record not existing yet.
 *	We don't want to clog the Fail log with useless data.
 * @result -1 if failed.  0 if success.
*/
function testRead ($ip, $apid, $noLogOnFail) {

	global $user_supplied_id;

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&user_supplied_id=$user_supplied_id&apid=$apid&rtn_format=NQV&action=read' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;

	exec ($command, $output);
	if ( $noLogOnFail == "true" ) {
		return successful($output);
	}
	logResult ($output);
	if (successful($output) == 0) {
		return testDataset("Read", $output, "" );
	}
	return -1;
}


/**
 * Given the 5 minute APID and IP address, test a delete of data for $user_supplied_id.
 * Results are logged.  Function returns -1 if failed, or 0 if success.
 * @param IP address
 * @param Five munite APID given for IP address.
 * @result -1 if failed.  0 if success.
*/
function testDelete ($ip, $apid) {

	global $user_supplied_id;

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&user_supplied_id=$user_supplied_id&apid=$apid&rtn_format=NQV&action=delete' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;

	exec ($command, $output);
	logResult ($output);
	return successful($output);
}


/**
 * Given the 5 minute APID and IP address, test the creation of data under the nickname "TestServer".
 * Results are logged.  Function returns -1 if failed, or 0 if success.
 * @param IP address
 * @param Five munite APID given for IP address.
 * @param The MAC specific template to use, retrieved from testTemplates.
 * @result -1 if failed.  0 if success.
*/
function testAdd ($ip, $apid, $template) {

	global $user_supplied_id;
	global $node_name;
	global $node_domain;
	global $mac_address;
	global $node_ip_address;
	global $node_subnet_mask;
	global $node_nameserver;
	global $node_default_gateway;
	global $enable_provisioning_flag;
	global $force_pxe_ip;

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&user_supplied_id=$user_supplied_id&mac_address=$mac_address&apid=$apid&control_file_template=$template&node_name=$node_name&node_domain=$node_domain&node_ip_address=$node_ip_address&node_subnet_mask=$node_subnet_mask&node_nameserver=$node_nameserver&node_default_gateway=$node_default_gateway&node_time_zone=Etc%2FGMT&enable_provisioning_flag=$enable_provisioning_flag&force_pxe_ip=$force_pxe_ip&rtn_format=NQV&action=add&node_password=unsecure' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	logResult ($output);

	if (successful($output) == 0) {
		return testDataset("Add", $output, "" );
	}
	return -1;
}



/**
 * Given the 5 minute APID and IP address, test the creation of data under the nickname "TestServer".
 * Results are logged.  Function returns -1 if failed, or 0 if success.
 * @param IP address
 * @param Five munite APID given for IP address.
 * @result -1 if failed.  0 if success.
*/
function testUpdate ($ip, $apid) {

	global $user_supplied_id;
	global $node_name_up;
	global $node_domain;
	global $mac_address;
	global $node_ip_address_up;
	global $node_subnet_mask_up;
	global $node_nameserver_up;
	global $node_default_gateway_up;
	global $enable_provisioning_flag_up;
	global $force_pxe_ip_up;

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&user_supplied_id=$user_supplied_id&mac_address=$mac_address&apid=$apid&node_name=$node_name_up&node_domain=$node_domain_up&node_ip_address=$node_ip_address_up&node_subnet_mask=$node_subnet_mask_up&node_nameserver=$node_nameserver_up&node_default_gateway=$node_default_gateway_up&node_time_zone=Etc%2FGMT&enable_provisioning_flag=$enable_provisioning_flag_up&force_pxe_ip=$force_pxe_ip_up&rtn_format=NQV&action=update&node_password=unsecure' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	logResult ($output);
	return successful($output);
}


///////////////////////////////////////
// Dataset Tests  Called by the Unit tests 
///////////////////////////////////////

/**
 * Check for good data after an add, read, or roles (maybe not roles) call.
 * If there is a mismatch, log the failure in detail.
 * @param The function calling us (Read, Add, etc)
 * @param The output of the last call.
 * @param The key prefix.  Roles adds a MAC_SPECIFIC_ROLE_(foo)_ to it's results.
 * @return 0 if all compares are successful, 1 upon the first failure.
 * Unfortunately, almost the identical code will have to be written for Update.
 */
function testDataset($function, $output, $keyPrefix ) {

	global $user_supplied_id;
	global $node_name;
	global $node_domain;
	global $node_ip_address;
	global $node_subnet_mask;
	global $node_nameserver;
	global $node_default_gateway;
	global $enable_provisioning_flag;
	global $force_pxe_ip;

	$key = $keyPrefix."user_supplied_id";
	if ( checkAndLogDataResult ($function, $output, $key, $user_supplied_id ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."node_name";
	if ( checkAndLogDataResult ($function, $output, $key, $node_name ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."node_domain";
	if ( checkAndLogDataResult ($function, $output, $key, $node_domain ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."node_ip_address";
	if ( checkAndLogDataResult ($function, $output, $key, $node_ip_address ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."node_subnet_mask";
	if ( checkAndLogDataResult ($function, $output, $key, $node_subnet_mask ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."node_nameserver";
	if ( checkAndLogDataResult ($function, $output, $key, $node_nameserver ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."node_default_gateway";
	if ( checkAndLogDataResult ($function, $output, $key, $node_default_gateway ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."enable_provisioning_flag";
	if ( checkAndLogDataResult ($function, $output, $key, $enable_provisioning_flag ) != 0 ) {
		return -1;
	}
	$key = $keyPrefix."force_pxe_ip";
	if ( checkAndLogDataResult ($function, $output, $key, $force_pxe_ip ) != 0 ) {
		return -1;
	}


	return 0;
}

/**
 *
 */
function checkAndLogDataResult ($function, $output, $key, $expectedValue ) {

	$value = getResultValue($output, $key);
	if ( strcmp ($expectedValue, $value) != 0 ) {
		logFail("While doing a ".$function." the key ".$key." returned '".$value.
			"' instead of '".$expectedValue."'");
		return -1;
	}
	return 0;
}







///////////////////////////////////////
// Functions 
///////////////////////////////////////

/**
 * Log info returned form the given command executed
 * @param Output from the command executed.  Details will be logged to the appropriate log file
 */
function logResult ($output) {

	$ar = getResultValue($output, "actionResults");
	$arm = getResultValue($output, "actionResultsMsg");
	$message = "actionResults: ".$ar." actionResultsMsg: ".$arm;
	echo ($message."\n");
	if ( successful($output) == 0 ) {
		logSuccess($message);
	} else {
		logFail($message);
	}
}

/**
 * Log a message to the success log.
 * @param Message to log.  It will be timestamped and posted to a timestamped log file.
 */
function logSuccess($message) {
	
	global $now;
	$logstamp = date("Y-m-d_H:i:s");
	$logitCmd = "echo "."\"".$logstamp." ".$message."\""." >> /usr/local/linmin/".$now."APITestSuccess.log";
	system($logitCmd);
}

/**
 * Log a message to the fail log.
 * @param Message to log.  It will be timestamped and posted to a timestamped log file.
 */
function logFail($message) {

	global $now;
	$logstamp = date("Y-m-d_H:i:s");
	$logitCmd = "echo "."\"".$logstamp." ".$message."\""." >> /usr/local/linmin/".$now."APITestFail.log";
	system($logitCmd);
}

/**
 * Move all existing APITest logs to the APITest directory
 */
function moveLogs() {
	system("mkdir -p /usr/local/linmin/APILogs");
	system("mv  /usr/local/linmin/*APITest*.log /usr/local/linmin/APILogs");
}



/**
 * Check to see if the given API operation was successful.  If it was, return 0.
 * @param The result of the given operation.
*/
function successful ($output ) {

	$result = getResultValue ($output, "actionResults" );
	if ( strcmp( $result, "OK") == 0 ) {
		return 0;
	}
	return -1;
}

/**
 * Given the resulting output of an API command in name=value (array) form, 
 * retrieve the value for a given key.
 * @param The output array
 * @param The key
 * @return The value associated with the given key.
*/
function getResultValue ($output, $key ) {

	foreach ($output as $q ) {
		$result = explode("=", $q);
		if ( strcmp($result[0], $key) == 0 ) {
			$result[1] = trim($result[1], "\"");
			$result[1] = rtrim($result[1], "\"");
			return $result[1];
		}
	}
	return "";
}

/**
 * Get the globalAPID for the buiilt-in user / password.
 * @return globalAPID
 */
function getGlobalAPID () {

	$apiScriptUrl = "http://127.0.0.1/tftpboot/www/lbmp-API.php?";
	$apiScript = "/home/tftpboot/www/lbmp-API.php";
	$apiApidScript = "php /home/tftpboot/www/lbmp-API_APID_management.php";
	$apiAuthPid = shell_exec($apiApidScript." allow examples | grep 'linminapi' | awk '{print $6}'");
	return $apiAuthPid;

}


?>
