<?php


include_once "ProvisionDataClass.php";
include_once "ProvisionRoleListClass.php";
include_once "UserClass.php";
include_once "Functions.php";
include_once "LoggerClass.php";

$loggerObject = new LoggerClass("APIProvisionTest");

// Test user_supplied_id to be used for all provisioning API tests
$user_supplied_id = "TestServer";


///////////////////////////////////////
// Run Workflows 
///////////////////////////////////////

/*
 Authentication tests...
 * authorize with base APID
 * re-authorize with current temp APID
 * re-authorize with base APID
 */
echo ("Attempting to authenticate. \n");
$userObject = new UserClass("127.0.0.1", "linminapi", "baremetalapi");
$loggerObject->logResult ($userObject);
if ( $userObject->actionResultOk != true ) {
	$loggerObject->logFail ("Initial Authentication failed. Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ("Initial Authentication successful. \n");

echo ("Attempting to reauth using the temp ID given by the last auth \n");
$base_apid = $userObject->user_apid;
$userObject->user_apid = $userObject->temp_apid;
$userObject->reAuth();
$loggerObject->logResult ($userObject);
if ( $userObject->actionResultOk != true ) {
	$loggerObject->logFail ("ReAuthentication with temp ID failed. Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ("Authentication using temp apid instead of base successful. \n");

echo ("Attempting to reauth using the original base ID \n");
$userObject->user_apid = $base_apid;
$userObject->reAuth();
$loggerObject->logResult ($userObject);
if ( $userObject->actionResultOk != true ) {
	$loggerObject->logFail ("ReAuthentication with original base ID. Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ("Authentication using original base ID successful. \n");

// Extracting templates.  This code is still old.
// We will just use the same template for everything.
$template = testTemplates();
if ( $template == -1 ) {
	echo ("Could not get template #1.  You must create at least one template.\n");
	echo ("Aborting tests\n");
	exit ($templates_failed);
}

/*
 Read role.  If it does exist, then delete.  Log success/fail on delete only
 */
echo ("Attempting to read role: ".$user_supplied_id." We will not log this result.\n");
$roleObject1 = new ProvisionDataClass($userObject, $user_supplied_id);
$roleObject1->read("provision");
if ( $roleObject1->actionResultOk == true ) {
	$loggerObject->logSuccess ($roleObject1->actionResultString."\n");
	echo ("The role exists  Testing delete \n");
	$roleObject1->delete("provision");
	$loggerObject->logResult ($roleObject1);
	if ( $roleObject1->actionResultOk != true ) {
		$loggerObject->logFail ("Failed to delete existing role.  Exiting\n");
		exit (0);
	}
	$loggerObject->logSuccess ("Role deleted successfully. \n");
}

/*
 Add new role, read role back, compare data
 */
echo ("Role is known to not exist.  Testing add then read back and compare. \n");
$roleObject1->add($template);
$loggerObject->logResult ($roleObject1);
if ( $roleObject1->actionResultOk != true ) {
	$loggerObject->logFail ("Failed to add a record that is known to not exist. Exiting\n");
	exit (0);
}
$roleObject2 = new ProvisionDataClass($userObject, $user_supplied_id);
$roleObject2->read("provision");
$loggerObject->logResult ($roleObject2);
if ( $roleObject2->actionResultOk != true ) {
	$loggerObject->logFail ("Failed to read a record that is known to exist. Exiting\n");
	exit (0);
}
$roleObject2->compareData($roleObject1);
if ( $roleObject2->equal != true ) {
	$loggerObject->logFail("After an Add, then read, the following data did not match. \n".$roleObject2->compareString);
	exit (0);
}
$loggerObject->logSuccess("Add then Read data matched\n");
	
/*
 Add a duplicate.  Should fail
 */
echo ("Role is known to exist.  Testing add to see if duplicate fails \n");
$roleObject1->add($template);
if ( $roleObject1->actionResultOk == true ) {
	$loggerObject->logFail ($roleObject1->actionResultString);
	$loggerObject->logFail ("Add a duplicate record returned OK.  This is wrong.. Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ($roleObject1->actionResultString);
$loggerObject->logSuccess ("Add a duplicate record returned FAIL.  This is expected\n");

/*
 Update existing role, read data back, compare
 */
echo ("Testing update then read back and compare data. \n");

$roleObject1->setAlternateValues();  // This call changes several values in the object for update testing.
$roleObject1->update($template);
$loggerObject->logResult ($roleObject1);
if ( $roleObject1->actionResultOk != true ) {
	$loggerObject->logFail ("Failed to update a record that is known to exist. Exiting\n");
	exit (0);
}
$roleObject2->read("provision");     // re read.  Should be the same as alternateValues
$loggerObject->logResult ($roleObject2);
if ( $roleObject2->actionResultOk != true ) {
	$loggerObject->logFail ("Failed to read a record that is known to exist. Exiting\n");
	exit (0);
}
$roleObject2->compareData($roleObject1);
if ( $roleObject2->equal != true ) {
	$loggerObject->logFail("After an Update, then read, the following data did not match. \n".$roleObject2->compareString);
	exit (0);
}
$loggerObject->logSuccess("Update then Read data matched\n");

/*
 Update roleObject1 to different MAC address.  Should fail.
 */
echo ("Changing the mac address on an existing role.  Should fail. \n");
$roleObject1->mac_address="aa:bb:cc:dd:ee:ff";
$roleObject1->update($template);
if ( $roleObject1->actionResultOk == true ) {
	$loggerObject->logFail ($roleObject1->actionResultString);
	$loggerObject->logFail ("Update different MAC address in existing Role returned OK.  This is wrong.. Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ($roleObject1->actionResultString);
$loggerObject->logSuccess ("Update to different MAC returned FAIL.  This is expected\n");

/*
 Read back roleObject1  We know this works.
 */
$roleObject1->read("provision");

/*
 Read the role list.  If successful see if roleObject1 exists in the list
 */
echo ("Reading Provisioning Role List... \n");
$provisioningRoleList = new ProvisionRoleListClass($userObject);
$provisioningRoleList->getRoles();
$loggerObject->logResult ($provisioningRoleList);
if ( $provisioningRoleList->actionResultOk != true ) {
	$loggerObject->logFail ("Failed to get role list.  Exiting.\n");
	exit (0);
}
echo ("Role list is... \n".$provisioningRoleList->toString());
if ( $provisioningRoleList->isRoleInList ($roleObject1) == false ) {
	$loggerObject->logFail ("Failed to find a role that is known to exist in the role list.  Exiting.\n");
	exit (0);
}
$loggerObject->logSuccess ("Found our test role in the role list.\n");

/*
 Finally delete and confirm that the role is gone.
 */
echo ("The role exists  Testing delete (again)\n");
$roleObject1->delete("provision");
$loggerObject->logResult ($roleObject1);
if ( $roleObject1->actionResultOk != true ) {
	$loggerObject->logFail ("Failed to delete existing role.  Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ("Role deleted successfully. \n");
$roleObject1->read("provision");
if ( $roleObject1->actionResultOk == true ) {
	$loggerObject->logFail ("Was able to read a deleted role.  Exiting\n");
	exit (0);
}
$loggerObject->logSuccess ("Was not able to read a deleted role.  \n");

echo ("All tests succeeded. \n");
exit (0);


///////////////////////////////////////
// Misc 
///////////////////////////////////////


/**
 * Given the global APID, user and password, request a 5 minute APID to be used for the unit
 * tests.  This function is a unit test as well.  If it fails, all other tests will be aborted.
 * @param The global APID
 * @param USer name
 * @param Passwrod
 * @result A valid APID or -1 if it failed.  Results are logged.
*/
function testTemplates() {

	global $userObject;
	global $loggerObject;

	$ip = $userObject->getIP();
	$apid = $userObject->getTempAPID();

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&apid=$apid&rtn_format=NQV&action=templates' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	$loggerObject->logOutput ($output);

	if (  successful($output) == 0) {
		$template = getResultValue ($output, "MAC_SPECIFIC_TEMPLATE_1_title" );
		echo ("Template 1 is: ".$template."\n");
		return $template;
	}
	return "-1";
}

/**
 * Check for good data after an add, read, or update call.
 * If there is a mismatch, log the failure in detail.
 * @param The function calling us (Read, Add, Update)
 * @param A readClass instance returned by some operation.
 * @param A readClass instance containing the expected values.
 * @return 0 if all compares are successful, 1 if any failed.
 */
function testDataset($function, $readObject1, $readObject2 ) {

	$readObject2->compareData($readObject1);
	if ( $readObject2->equal == true ) {
		return 0;
	}

	logFail("While doing a ".$function." the following key value paris did not match \n".$readObject2->compareString);
	return -1;
}


/**
 * Check to see if the given API operation was successful.  If it was, return 0.
 * @param The result of the given operation.
*/
function successful ($output ) {

	$result = getResultValue ($output, "actionResults" );
	if ( strcmp( $result, "OK") == 0 ) {
		return 0;
	}
	return -1;
}



?>
