<?php


include_once "ProvisionDataClass.php";
include_once "ProvisionRoleListClass.php";
include_once "UserClass.php";
include_once "Functions.php";

// Test user_supplied_id to be used for all provisioning API tests
$user_supplied_id = "TestServer";
$control_file_template = "";

echo ("Welcome to the interactive Provisioning Role test tool\n\n");



// Authenticate
echo ("Attempting to authenticate...\n");
$userObject = $userObject = new UserClass("127.0.0.1", "linminapi", "baremetalapi");
if ( $userObject->actionResultOk != true ) {
	echo ("Authentication failed \n");
	echo ($userObject->actionResultString."\m");
	echo ("Exiting");
	exit (0);
}
echo ("Authentication successful.  You have five minutes. \n\n");

echo ("Reading Provisioning Role List... \n");
$provisioningRoleList = new ProvisionRoleListClass($userObject);
$provisioningRoleList->getRoles();
if ( $provisioningRoleList->actionResultOk != true ) {
	echo ("Failed to get role list...\n");
	echo ($provisioningRoleList->actionResultString."\m");
	echo ("Exiting");
	exit (0);
}
echo ( $provisioningRoleList->toString() );
$listLength = $provisioningRoleList->getRoleListLength();
echo ("The number of roles in the list is: ".$listLength."\n");
if ( $listLength > 0 ) {
	echo ("Using first role in the list\n");
	$provisioningRole = $provisioningRoleList->getRole(0);
	$user_supplied_id = $provisioningRole->user_supplied_id;
	$control_file_template = $provisioningRole->control_file_template;
} else {
	echo ("No roles exist.  Creating a new role called ".$user_supplied_id."\n");
	$provisioningRole = new ProvisionDataClass($userObject, $user_supplied_id);
}
echo ("Now working with role ".$user_supplied_id."\n\n");

while ( true ) {

	echo ("\n");
	echo ("Would you like to change the User Supplied ID (nick name in the GUI) ?\n");
	echo ("You are currently working with ID: ".$user_supplied_id."\n");
	echo ("If you change the ID, this program will attempt to read the associated role data.\n");
	echo ("If a read is unsuccessful becuase the role does not exits, \n");
	echo ("sensable default values will be set.\n");
	$value = InputText ("Change user supplied id (y/n)", "n");
	if ( strcmp( $value, "y") == 0 ) {
		$user_supplied_id = inputText ("Enter new user supplied id", $user_supplied_id);

		echo ("Attempting to read role ".$user_supplied_id."\n");
		$provisioningRole = new ProvisionDataClass($userObject, $user_supplied_id);
		$provisioningRole->read("image");
		echo ($provisioningRole->actionResultString."\n");
		if ($provisioningRole->actionResultOk == true) {
			echo ("The read was successful \n");
			$control_file_template = $provisioningRole->control_file_template;
		} else {
			echo ("The read failed, see above message. \n");
		}
		inputText("Hit return to continue", "");
	}

	echo ("------------------------------------------------------------\n");
	echo ("Current contents of profile ".$user_supplied_id."\n");
	print_r ($provisioningRole->toString() );
	echo ("------------------------------------------------------------\n\n");

	echo ("If you know the profile does not exist, choose Add.\n");
	echo ("If you know the profile does exist, choose Update or Delete..\n");

	$value = inputText ("Add a: Update u: Delete d: Exit e: ", "a");


	if ( strcmp( $value, "e") == 0 ) {
		echo ("Exiting... \n" );
		exit (0);
	}
	if ( strcmp( $value, "a") == 0 ) {
		echo ("Add \n" );
		modifyProfile();
		echo ("Attempting to add...\n");
		$provisioningRole->add($control_file_template);
		echo ($provisioningRole->actionResultString."\n");
	}
	if ( strcmp( $value, "u") == 0 ) {
		echo ("Update \n" );
		modifyProfile();
		echo ("Attempting to update...\n");
		$provisioningRole->update($control_file_template);
		echo ($provisioningRole->actionResultString."\n");
	}
	if ( strcmp( $value, "d") == 0 ) {
		$d = inputText("Are you sure you want to delete (y/n)?", "n");
		if ( strcmp( $d, "y") == 0 ) {
			echo ("Attempting to update...\n");
			$provisioningRole->delete("provision");
			echo ($provisioningRole->actionResultString."\n");
		}
	}

	$value = inputText ("Exit (y/n) ", "n");
	if ( strcmp( $value, "y") == 0 ) {
		echo ("Exiting... \n" );
		exit (0);
	}
}

exit (0);

function modifyProfile () {

	global $provisioningRole;
	global $control_file_template;
	echo ("Verify the following data for Add or Update...\n");

	// A slightly special case for control file template
	echo ("Control file template is currently set to:".$control_file_template."\n");
	$d = inputText("Do you want to change / set the control file template (y/n)?", "n");
	if ( strcmp( $d, "y") == 0 ) {
		$control_file_template = chooseTemplate();
	}

	$provisioningRole->node_name = inputText("node_name", $provisioningRole->node_name);
	$provisioningRole->node_password = inputText("node_password", $provisioningRole->node_password);
	$provisioningRole->node_domain = inputText("node_domain", $provisioningRole->node_domain);
	$provisioningRole->node_ip_address = inputText("node_ip_address", $provisioningRole->node_ip_address);
	$provisioningRole->node_subnet_mask = inputText("node_subnet_mask", $provisioningRole->node_subnet_mask);
	$provisioningRole->node_nameserver = inputText("node_nameserver", $provisioningRole->node_nameserver);
	$provisioningRole->node_default_gateway = inputText("node_default_gateway", $provisioningRole->node_default_gateway);
	$provisioningRole->enable_provisioning_flag = 
		inputText("enable_provisioning_flag (one of nextbootonly, ignore, liveubuntu)", 
		$provisioningRole->enable_provisioning_flag);
	$provisioningRole->mac_address = inputText("mac_address", $provisioningRole->mac_address);
	$d = inputText("Set force_pxe_ip to 'on' (y/n)?", "n");
	if ( strcmp( $d, "y") == 0 ) {
		$provisioningRole->force_pxe_ip = "on";
	} else {
		$provisioningRole->force_pxe_ip = "";
	}
}


/**
 * Get one of the available templates to set a new role to.
 * XXX This code is disgusting and will be replaced with a class.
*/
function chooseTemplate() {

	global $userObject;
	$ip = $userObject->getIP();
	$apid = $userObject->getTempAPID();

	$tmpfile = "tempwget.txt";
	$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=provision&apid=$apid&rtn_format=NQV&action=templates' http://$ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
	exec ($command, $output);
	
	echo ("Choose one of the following templates \n");
	print_r($output);
	echo ("Choose a template number: ");
	$number = fgets(STDIN);
	$number = rtrim($number);

	if (  strcmp ( getResultValue ($output, "actionResults" ) , "OK" ) == 0) {
		$template = getResultValue ($output, "MAC_SPECIFIC_TEMPLATE_".$number."_title" );
		echo ("Template 1 is: ".$template."\n");
		return $template;
	}
	return "-1";
}


?>
