<?php


include_once "ReadClass.php";


class ProvisionDataClass extends ReadClass{

	// Test values for adding updating and reading
	// for the Provision set of tests.
	public $node_name = "TestHost";
	public $node_domain = "TestDomain";
	public $node_ip_address = "192.168.0.200";
	public $node_subnet_mask = "255.255.255.0";
	public $node_nameserver = "192.168.1.254";
	public $node_default_gateway = "192.168.1.1";
	public $enable_provisioning_flag = "nextbootonly";
	public $force_pxe_ip = "on";  // Write only, do not test for comparison.
	public $mac_address = "00:0c:29:14:CA:DC";
	public $node_time_zone = "Etc%2FGMT";
	public $control_file_template;   // Supplied by caller to Add.  Muse be existing template.
	public $node_password = "unsecure";
	
	// Read only values, do not use for comparason.
	public $pxe_event_ip;
	public $pxe_event_state;
	public $pxe_event_date_time;

	// Results from data compare between two ProvisionDataInstances.
	public $compareString = "";
	public $equal = true;

	/**
	 Once a successful read/add/update is called, this object will be updated.
	 This method is required by any extender of ReadClass.
	*/
	protected function setValues () {

		$this->node_name = $this->getOutputValue("node_name");
		$this->node_domain = $this->getOutputValue("node_domain");
		$this->node_ip_address = $this->getOutputValue("node_ip_address");
		$this->node_subnet_mask = $this->getOutputValue("node_subnet_mask");
		$this->node_nameserver = $this->getOutputValue("node_nameserver");
		$this->node_default_gateway = $this->getOutputValue("node_default_gateway");
		$this->enable_provisioning_flag = $this->getOutputValue("enable_provisioning_flag");
		$this->force_pxe_ip = $this->getOutputValue("force_pxe_ip");  // Write only, is probably blank
		$this->mac_address = $this->getOutputValue("mac_address");
		$this->node_time_zone  = $this->getOutputValue("node_time_zone");
		$this->control_file_template = $this->getOutputValue("control_file_template");
		$this->node_password = $this->getOutputValue("node_password");

		// read only, do not use in a compare
		$this->pxe_event_ip = $this->getOutputValue("pxe_event_ip");
		$this->pxe_event_state = $this->getOutputValue("pxe_event_state");
		$this->pxe_event_date_time = $this->getOutputValue("pxe_event_date_time");
	}

	/**
	 Check for equiv to another ReadData instance.  Store a string suitable for logging
	 if differences were found.  
	*/
	public function compareData( $that ) {

		$this->compareString = "";
		$this->equal = true;
		$cs = "\n";

		if ( strcmp ($this->node_name , $that->node_name) != 0 ) {
			$cs = $cs." the key 'node_name' returned '".$that->node_name.
			"' instead of '".$this->node_name."' \n";
			$this->equal = false;
		}
		if ( strcmp ($this->node_domain , $that->node_domain) != 0 ) {
			$cs = $cs." the key 'node_domain' returned '".$that->node_domain.
			"' instead of '".$this->node_domain."' \n";
			$this->equal = false;
		}
		if ( strcmp ($this->node_ip_address , $that->node_ip_address) != 0 ) {
			$cs = $cs." the key 'node_ip_address' returned '".$that->node_ip_address.
			"' instead of '".$this->node_ip_address."' \n";
			$this->equal = false;
		}
		if ( strcmp ($this->node_subnet_mask , $that->node_subnet_mask) != 0 ) {
			$cs = $cs." the key 'node_subnet_mask' returned '".$that->node_subnet_mask.
			"' instead of '".$this->node_subnet_mask."' \n";
			$this->equal = false;
		}
		if ( strcmp ($this->node_nameserver , $that->node_nameserver) != 0 ) {
			$cs = $cs." the key 'node_nameserver' returned '".$that->node_nameserver.
			"' instead of '".$this->node_nameserver."' \n";
			$this->equal = false;
		}
		if ( strcmp ($this->node_default_gateway , $that->node_default_gateway) != 0 ) {
			$cs = $cs." the key 'node_default_gateway' returned '".$that->node_default_gateway.
			"' instead of '".$this->node_default_gateway."' \n";
			$this->equal = false;
		}
		if ( strcmp ($this->nable_provisioning_flag , $that->nable_provisioning_flag) != 0 ) {
			$cs = $cs." the key 'nable_provisioning_flag' returned '".$that->nable_provisioning_flag.
			"' instead of '".$this->nable_provisioning_flag."' \n";
			$this->equal = false;
		}

		$this->compareString = $cs;

	}

	/**
	 A convenience method for setting a set of alternate values for testing Update.
	 */
	public function setAlternateValues () {
		$this->node_name = "TestHostUpdated";
		$this->node_domain = "TestDomainUpdated";
		$this->node_ip_address = "10.0.0.200";
		$this->node_subnet_mask = "255.0.0.0";
		$this->node_nameserver = "10.168.1.254";
		$this->node_default_gateway = "10.0.0.1";
		$this->enable_provisioning_flag = "ignore";
		$this->force_pxe_ip = "on";

	}

	public function add ( $control_file_template ) {
		$this->action = "add";
		$this->actiontype = "provision";
		$this->control_file_template = $control_file_template;
		$this->write();
	}

	public function update ( ) {
		$this->action = "update";
		$this->actiontype = "provision";
		$this->write();
	}

	/**
	 Either Add or Update depending on actiontype.
	 If the operation is successful, this object's class member vars will be updated.
	 If not, it will retain the previous set of values.
	*/
	private function write () {

		$mac_address = str_replace(":", "%3A", $this->mac_address);
		$tmpfile = "tempwget.txt";
		$command = <<< EOT
wget -qO $tmpfile --post-data 'actiontype=$this->actiontype&user_supplied_id=$this->user_supplied_id&mac_address=$mac_address&apid=$this->apid&control_file_template=$this->control_file_template&node_name=$this->node_name&node_domain=$this->node_domain&node_ip_address=$this->node_ip_address&node_subnet_mask=$this->node_subnet_mask&node_nameserver=$this->node_nameserver&node_default_gateway=$this->node_default_gateway&node_time_zone=$this->node_time_zone&enable_provisioning_flag=$this->enable_provisioning_flag&force_pxe_ip=$this->force_pxe_ip&rtn_format=$this->rtn_format&action=$this->action&node_password=$this->node_password' http://$this->ip/tftpboot/www/lbmp-API.php; cat $tmpfile; rm -f $tmpfile
EOT;
		$output = array();
		exec ($command, $output);
		$this->setActionResults($output);
		if ( $this->actionResultOk == true ) {
			$this->setValues();
		}

	} 

} // End of class

	/**
	 This is a unit test to check the equiv function
	 */
	function test () {
		$instance1 = new ProvisionDataClass();
		$instance1->compareData($instance1);
		echo ("It sould be equal to istelf \n");
		echo ( "instance1->equal = ".$instance1->equal."  this means ");
		if ( $instance1->equal == true ) {
			echo "true\n";
		} else {
			echo "false\n";
		}

		$instance2 = new ProvisionDataClass();
		$instance2->setAlternateValues();
		$instance2->compareData($instance1);
		echo ("It sould not be equal to instance2 \n");
		echo ( "instance2->equal = ".$instance2->equal."  this means ");
		if ( $instance2->equal == false ) {
			echo "false\n";
		} else {
			echo "true\n";
		}
		echo ( "instance2->compareString = ".$instance2->compareString."\n");
	}	




?>
